SELECT	DISTINCT [EventName], SatNum
FROM	[PASSData].[dbo].[SQLSatRegistrations]
WHERE	EventName like '%Richmond%'
ORDER BY SatNum;

SELECT	DISTINCT [EventName], SatNum
FROM	[PASSData].[dbo].[SQLSatRegistrations]
WHERE	EventName like '%Washington%'
ORDER BY SatNum;

/* 
Wayne has access to:
187
277
381
486

Chris has access to:
173
233
347
470

Karla has access to all:
*/

-- Add the users:
DROP USER IF EXISTS KARLA;
CREATE USER KARLA WITHOUT LOGIN;
GRANT SELECT ON dbo.SQLSatRegistrations TO [KARLA];

DROP USER IF EXISTS CHRIS;
CREATE USER CHRIS WITHOUT LOGIN;
GRANT SELECT ON dbo.SQLSatRegistrations TO [CHRIS];

DROP USER IF EXISTS WAYNE;
CREATE USER WAYNE WITHOUT LOGIN;
GRANT SELECT ON dbo.SQLSatRegistrations TO [WAYNE];

DROP USER IF EXISTS RAVI;
CREATE USER RAVI WITHOUT LOGIN;
GRANT SELECT ON dbo.SQLSatRegistrations TO [RAVI];
GO

DROP SECURITY POLICY IF EXISTS dbo.SQLSatFilter;
GO
DROP FUNCTION IF EXISTS dbo.fSQLSATFilter;
GO

CREATE FUNCTION dbo.fSQLSATFilter(@SatNum INTEGER)
RETURNS TABLE      -- must be an inline table-valued function
WITH SCHEMABINDING -- must have the SCHEMABINDING clause
AS
RETURN	SELECT	1 AS SQLSATFilter
		WHERE	CURRENT_USER IN ('KARLA','dbo') 
		OR		(CURRENT_USER = 'CHRIS' AND @SatNum IN (173,233,347,470))
		OR		(CURRENT_USER = 'WAYNE' AND @SatNum in (187,277,381,486));
GO

CREATE SECURITY POLICY dbo.SQLSatFilter
ADD FILTER PREDICATE dbo.fSQLSATFilter(SatNum) -- user FILTER to filter out rows for read operations; use BLOCK to block write operations
ON dbo.SQLSatRegistrations -- target table
WITH (STATE = ON);
GO

SELECT CURRENT_USER, * FROM SQLSatRegistrations;
GO

EXECUTE AS USER = 'KARLA';
GO
SELECT CURRENT_USER, * FROM SQLSatRegistrations;
GO
REVERT;
GO


EXECUTE AS USER = 'CHRIS';
GO
SELECT CURRENT_USER, * FROM SQLSatRegistrations;
GO
REVERT;
GO


EXECUTE AS USER = 'WAYNE';
GO
SELECT CURRENT_USER, * FROM SQLSatRegistrations;
GO
REVERT;
GO


EXECUTE AS USER = 'RAVI';
GO
SELECT CURRENT_USER, * FROM SQLSatRegistrations;
GO
REVERT;
GO
