USE tempdb;
GO
IF EXISTS (SELECT 1 FROM sys.tables where object_id = OBJECT_ID('dbo.Employee') and temporal_type <> 0)
	ALTER TABLE dbo.Employee SET (SYSTEM_VERSIONING = OFF);
IF EXISTS (select * from sys.periods where object_id = OBJECT_ID('dbo.Employee'))
	ALTER TABLE dbo.Employee DROP PERIOD FOR SYSTEM_TIME;
DROP TABLE IF EXISTS dbo.EmployeeHistory;
DROP TABLE IF EXISTS dbo.Employee;
CREATE TABLE dbo.Employee (  
	EmployeeID		INTEGER IDENTITY NOT NULL PRIMARY KEY CLUSTERED, 
	Name			NVARCHAR(100) NOT NULL, 
	Position		VARCHAR(100) NOT NULL , 
	Department		VARCHAR(100) NOT NULL, 
	Address			NVARCHAR(1024) NOT NULL, 
	AnnualSalary	DECIMAL (10,2) NOT NULL, 
	ValidFrom		DATETIME2 (2) GENERATED ALWAYS AS ROW START /*HIDDEN*/, -- HIDDEN: column doesn't show up in a SELECT *. Must reference column for it to be in result set.
	ValidTo			DATETIME2 (2) GENERATED ALWAYS AS ROW END /*HIDDEN*/, 
	PERIOD FOR SYSTEM_TIME (ValidFrom, ValidTo)
	)  
WITH (SYSTEM_VERSIONING = ON (HISTORY_TABLE = dbo.EmployeeHistory)); 
-- By default, the history table is PAGE compressed.
-- If history table not specified, one will be created named MSSQL_TemporalHistoryFor_<base table object id>
GO
SELECT * FROM sys.tables where name like 'Employee%';
GO

SELECT * FROM dbo.Employee;
SELECT * FROM dbo.EmployeeHistory;
SELECT 'Empty Tables', GETUTCDATE();
GO
INSERT INTO dbo.Employee (Name, Position, Department, Address, AnnualSalary)
VALUES	('Wayne', 'DBA', 'IT', '123 Nonyabiz', 250000),
		('Chris', 'DBA', 'IT', '123 Nonyabiz', 250000);
GO
SELECT * FROM dbo.Employee;
SELECT * FROM dbo.EmployeeHistory;
SELECT 'After Insert', GETUTCDATE();
GO
waitfor delay '00:00:02';
UPDATE dbo.Employee SET AnnualSalary += 50000;
GO
SELECT * FROM dbo.Employee;
SELECT * FROM dbo.EmployeeHistory;
SELECT 'After Update', GETUTCDATE();

GO

/*

SELECT * FROM dbo.Employee FOR SYSTEM_TIME ALL;

DECLARE @Time1 DATETIME = '2016-05-24 00:11:21.413', -- Initial time
		@Time2 DATETIME = '2016-05-24 00:11:21.563', -- time after insert
		@Time3 DATETIME = '2016-05-24 00:11:23.830'; -- time after update
SELECT * FROM dbo.Employee FOR SYSTEM_TIME BETWEEN @Time1 AND @Time2;
SELECT * FROM dbo.Employee FOR SYSTEM_TIME BETWEEN @Time1 AND @Time3;
SELECT * FROM dbo.Employee FOR SYSTEM_TIME BETWEEN @Time2 AND @Time3;

SELECT * FROM dbo.Employee FOR SYSTEM_TIME AS OF @Time2;

AS OF        - The records that were active at the specified point-in-time.
BETWEEN      - Record versions active at any point during the specified start/end time periods (inclusive).
FROM         - Record versions active at any point during the specified start/end time periods. 
			   Includes rows that became active exactly on the lower boundary, but not those that
			   became active exactly on the upper boundary.
CONTAINED IN - Record versions that were opened and during the specified time periods. 
		       Includes rows that became active exactly on the lower boundary, or ceased being
		       active exactly on the upper boundary.
*/

